/*
 *  eb60_board.c
 *  Board support for the GB60 board.
 *
 *  Created by Stefan Robl on 21.01.06.
 *  Copyright 2006 Stefan Robl <stefan@qdev.de>. All rights reserved.
 *
 */
 
#include "emTypes.h"
#include "gb60_board.h"
#include "derivative.h" // include peripheral declarations

#define GB60_TIMERFLAG_ENABLED  0x01
#define GB60_TIMERFLAG_FINISHED 0x02

#define GB60_PWM_ISR_FREQUENCY  (4000000/8/256)
#define GB60_TIMER_DIVISOR      2

static uint16 timerCounters[GB60_NUMBER_OF_TIMERS];
static uint8  timerFlags[GB60_NUMBER_OF_TIMERS];
static uint8  pwmValues[GB60_NUMBER_OF_PWMS];
static uint8  timerDivisior = 0;
static uint8  lastReceivedChar = 0;


// **** SERIAL ****************************************************************

interrupt 17 void isrSerialRX(void)
{
	if (SCI1S1_RDRF)
	{
		lastReceivedChar = SCI1D;
	}
}

void gb60_serialInit(void)
{
	lastReceivedChar = 0;
 
	SCI1BDH = 0;
    SCI1BDL = 13;  // 19200 baud
    SCI1C1  = 0x00;
    SCI1C2  = SCI1C2_RIE_MASK | SCI1C2_TE_MASK | SCI1C2_RE_MASK;
}

void gb60_serialPutC(char c)
{
    uint8 dummy;
    
	if (c == '\n')
	{
	    dummy = SCI1S1;
	    SCI1D = '\r';
	    while (!SCI1S1_TC) { }
	}
	
	dummy = SCI1S1;
    SCI1D = c;
    while (!SCI1S1_TDRE) { }
    
    while (!SCI1S1_TC) { }
}

char gb60_serialGetC(void)
{
	char c = lastReceivedChar;
	lastReceivedChar = 0;
	return c;
}

void gb60_serialPutStr(const char *str)
{
	if (str)
	{
	    uint8 c = SCI1S1;
	    
	    while (1)
	    {
	        c = *str++;
	        if (!c)
	        {
	            break;
	        }
	        else if (c == '\n')
	        {
    	        SCI1D = '\r';
                while (!SCI1S1_TDRE) { }   
	        }
	        
            SCI1D = c;
            while (!SCI1S1_TDRE) { }
	    }
	    
	    while (!SCI1S1_TC) { }
	}
}


// **** LEDs ******************************************************************

void gb60_initLEDs(void)
{
    PTFDD = GB60_LEDS_MASK; // enable LEDs output
}

void gb60_setLEDs(uint8 state, uint8 mask)
{
    PTFD = (PTFD & (~(mask & GB60_LEDS_MASK))) | (state & GB60_LEDS_MASK);
}

void gb60_toggleLEDs(uint8 leds)
{
    PTFD ^= (leds & GB60_LEDS_MASK);
}

// **** ADCs ******************************************************************

void gb60_initADC(uint8 mask)
{
    ATD1C  = ATD1C_ATDPU_MASK | ATD1C_DJM_MASK | 0x1;
    ATD1PE = mask;
}

uint16 gb60_readADC(uint8 adcNr)
{
    ATD1SC = adcNr & ATD1SC_ATDCH_MASK;
    
    while (!ATD1SC_CCF)
    {
    }
    
    return (((uint16)ATD1RH)<<8) | ((uint16) ATD1RL);
}

uint16 gb60_readADCSampled(uint8 adcNr, uint8 samples)
{
    uint32 avg = 0;
    uint8 c;
    
    if (!samples)
    {
        samples = 1;
    }
    
    do   
    {
        avg += gb60_readADC(adcNr);
    }
    while (--c);

    avg /= samples; 
    
    return (uint16) avg;
}

// **** frequency *************************************************************

void gb60_initFrequency(void)
{
    TPM1SC   = TPM1SC_CLKSA_MASK; 
    TPM1C0SC = 0x00;
}

void gb60_setFrequency(uint16 frequency)
{
    uint16 val;
    
    if (frequency)
    {
        val = (uint16) ((4000000 / 2) / (uint32)frequency);
        TPM1MODH = (val>>8) & 0xff;
        TPM1MODL = val & 0xff;
        
        if ((TPM1C0SC & 0x3f) != 0x14)
        {
            TPM1C0SC = 0x14;
        }
    }
    else
    {
        TPM1C0SC = 0x00;
    }
}

// **** TIMERs & PWMs *********************************************************

// timer base clock needs to be at least 1ms / 256 = 256 kHz

interrupt 14 void timerISR(void) // called with 1953,125 Hz (every 512s)
{
    uint8 c, t;

    if (TPM2SC_TOF)
    {
        TPM2C0VH = 0;
        TPM2C1VH = 0;
        TPM2C2VH = 0;
        TPM2C3VH = 0;
        TPM2C4VH = 0;
        
        TPM2C0VL = pwmValues[0];
        TPM2C1VL = pwmValues[1];
        TPM2C2VL = pwmValues[2];
        TPM2C3VL = pwmValues[3];
        TPM2C4VL = pwmValues[4];
        
        if (!timerDivisior)
        {
            c = GB60_NUMBER_OF_TIMERS;
            if (c)
            {
                do
                {
                    c--;
                    
                    t = timerFlags[c];
                    if (t & GB60_TIMERFLAG_ENABLED)
                    {
                        if (!timerCounters[c])
                        {
                            t |=  GB60_TIMERFLAG_FINISHED;
                            t &= ~GB60_TIMERFLAG_ENABLED;
                            timerFlags[c] = t;
                        }   
                        timerCounters[c]--; 
                    }    
                }
                while (c);
            }
            timerDivisior = GB60_TIMER_DIVISOR;
        }
        timerDivisior--;
           
        TPM2SC_TOF = 0;
    }  
}


void gb60_init_PWM_Timer(void)
{
    uint8 c;
    
    for(c=0; c<GB60_NUMBER_OF_TIMERS; c++)
    {
        timerCounters[c] = 0; 
        timerFlags[c]    = GB60_TIMERFLAG_FINISHED; 
    }
    
    for(c=0; c<GB60_NUMBER_OF_PWMS; c++)
    {
        pwmValues[c] = 0;
    }
    
    timerDivisior = 0;
    
    TPM2CNTH = 0;
    TPM2CNTL = 0;
    TPM2MODH = 1; // modulo 256
    TPM2MODL = 0;
    TPM2SC   = TPM2SC_TOIE_MASK | TPM2SC_CLKSA_MASK | 0x3; // divisor=8: 4000000/8/256 = 1953,125 Hz isr frequency 
    
    TPM2C0SC = 0x00;
    TPM2C1SC = 0x00;
    TPM2C2SC = 0x00;
    TPM2C3SC = 0x00;
    TPM2C4SC = 0x00;
}

// **** TIMER *************************

void gb60_startTimer(uint8 timerNr, uint16 msDelay)
{
    if (timerNr < GB60_NUMBER_OF_TIMERS)
    {
        timerFlags[timerNr] &= ~(GB60_TIMERFLAG_ENABLED | GB60_TIMERFLAG_FINISHED);
        
        if (msDelay)
        {
            timerCounters[timerNr] = (uint16)( ( ((uint32)1000 * (uint32)GB60_TIMER_DIVISOR * (uint32)msDelay) + (GB60_PWM_ISR_FREQUENCY - 1) ) / (uint32)GB60_PWM_ISR_FREQUENCY );
            timerFlags[timerNr] |=  GB60_TIMERFLAG_ENABLED;
        }
        else
        {
            timerFlags[timerNr] |= GB60_TIMERFLAG_FINISHED;
        }
    }
}

void gb60_stopTimer(uint8 timerNr)
{
    if (timerNr < GB60_NUMBER_OF_TIMERS)
    {
        timerFlags[timerNr] = GB60_TIMERFLAG_FINISHED;
    }
}

uint16 gb60_getTimer_ms(uint8 timerNr)
{
    uint16 result;
    
    if (timerNr < GB60_NUMBER_OF_TIMERS)
    {
        do
        {
            // the ISR could alter the uint16 while the access is done
            result = timerCounters[timerNr];
        }
        while (result != timerCounters[timerNr]);
    }
    else
    {
        result = 0;
    }
    
    return result;
}

boolean gb60_checkTimerFinished(uint8 timerNr)
{
    boolean result = FALSE;
    
    if (timerNr < GB60_NUMBER_OF_TIMERS)
    {
        if (timerFlags[timerNr] & GB60_TIMERFLAG_FINISHED)
        {
            result = TRUE;
        }
    }
    
    return result;
}

void gb60_delayUsingTimer(uint8 timerNr, uint16 ms)
{
    if (timerNr < GB60_NUMBER_OF_TIMERS)
    {
        gb60_startTimer(timerNr, ms);
        while (!(timerFlags[timerNr] & GB60_TIMERFLAG_FINISHED))
        {
        }
    }
}

// **** PWM ***************************

void gb60_setPWMEnabled(uint8 pwmNr, boolean enable)
{
    if (pwmNr < GB60_NUMBER_OF_PWMS)
    {
        uint8 v;
        
        if (enable)
        {
            v = 0x28;   
        }
        else
        {
            v = 0;   
        }
        
        switch (pwmNr)
        {
            case 0:
                TPM2C0SC = v;
                break;
            case 1:
                TPM2C1SC = v;
                break;
            case 2:
                TPM2C2SC = v;
                break;
            case 3:
                TPM2C3SC = v;
                break;
            case 4:
                TPM2C4SC = v;
                break;
            default:
                break;
        }
    }
}

void gb60_setPWMValue(uint8 pwmNr, uint8 value)
{
    if (pwmNr < GB60_NUMBER_OF_PWMS)
    {
        pwmValues[pwmNr] = value;    
    }
}
